package sap.livelink365.callbacks.service;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Base64;
import java.util.Properties;
import org.json.*;

public class MessageService {

    private String baseUrl;
    private String appKey;
    private String appSecret;
    private String defaultOrigin;
    private Properties properties;

    public MessageService() throws IOException{
        properties = new Properties();
        InputStream input = null;
        ClassLoader loader = Thread.currentThread().getContextClassLoader();

        try {
            input = loader.getResourceAsStream("global.properties");
            // load a properties file
            properties.load(input);
        } catch (IOException ex) {
            ex.printStackTrace();
            throw ex;
        } finally {
            if (input != null) {
                try {
                    input.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        this.baseUrl = properties.getProperty("baseUrl");
        this.appKey = properties.getProperty("appKey");
        this.appSecret = properties.getProperty("appSecret");
        this.defaultOrigin = properties.getProperty("defaultOrigin");
    }

    public String getAccessToken() throws IOException
    {
        String token_url = this.baseUrl + "/oauth/token";
        String authenticationToken = this.appKey + ":" + this.appSecret;
        String basicToken = Base64.getEncoder().encodeToString(authenticationToken.getBytes());
        String param = "grant_type=client_credentials";
        URL url = new URL(token_url);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("POST");
        connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
        connection.setRequestProperty("Accept", "application/json");
        connection.setRequestProperty("Authorization", "Basic " + basicToken);
        connection.setDoOutput(true);
        DataOutputStream writer = new DataOutputStream(connection.getOutputStream());
        writer.writeBytes(param);
        writer.flush();
        writer.close();
        InputStreamReader inputStreamReader = new InputStreamReader(connection.getInputStream());
        BufferedReader reader = new BufferedReader(inputStreamReader);
        StringBuilder response = new StringBuilder();
        String inputLine;
        while ((inputLine = reader.readLine()) != null) response.append(inputLine);
        reader.close();
        JSONObject object = new JSONObject(response.toString());
        return object.getString("access_token");
    }

    public void sendMessage(String message, String destination, String origin) throws IOException
    {
        String messageUrl = this.baseUrl + "/v2/sms";
        String bearer = getAccessToken();
        JSONObject messagePayload = new JSONObject()
                .put("message", message)
                .put("origin", origin.isEmpty() ? this.defaultOrigin : origin)
                .put("destination", destination);
        URL url = new URL(messageUrl);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("POST");
        connection.setRequestProperty("Content-Type", "application/json; charset=UTF-8");
        if (bearer != null && !bearer.isEmpty())
        {
            connection.setRequestProperty("Authorization", "Bearer " + bearer);
        }
        connection.setDoOutput(true);
        DataOutputStream writer = new DataOutputStream(connection.getOutputStream());
        writer.writeBytes(messagePayload.toString());
        writer.flush();
        writer.close();
        InputStreamReader inputStreamReader = new InputStreamReader(connection.getInputStream());
        BufferedReader reader = new BufferedReader(inputStreamReader);
        StringBuilder response = new StringBuilder();
        String inputLine;
        while ((inputLine = reader.readLine()) != null) response.append(inputLine);
        reader.close();
        System.out.println(response.toString());
    }

}
